"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformMain = void 0;
const compiler_sfc_1 = require("@vue/compiler-sfc");
const index_1 = require("./index");
const querystring_1 = __importDefault(require("querystring"));
const descriptorCache_1 = require("./utils/descriptorCache");
const path_1 = __importDefault(require("path"));
async function transformMain(code, filePath, options, pluginContext) {
    const descriptor = descriptorCache_1.createDescriptor(code, filePath, options);
    const hasFunctional = descriptor.template && descriptor.template.attrs.functional;
    // template
    const { code: templateCode, templateRequest } = genTemplateRequest(filePath, descriptor);
    // script
    const scriptVar = 'script';
    const { scriptCode } = await genScriptCode(scriptVar, descriptor, filePath, options);
    // style
    const cssModuleVar = '__cssModules';
    const { scoped, stylesCode } = genStyleRequest(cssModuleVar, descriptor, filePath);
    let result = `${scriptCode}
${templateCode}
const ${cssModuleVar} = {}
${stylesCode}
/* normalize component */
import normalizer from "${index_1.vueComponentNormalizer}"
var component = normalizer(
  script,
  render,
  staticRenderFns,
  ${hasFunctional ? `true` : `false`},
  injectStyles,
  ${scoped ? JSON.stringify(descriptor.id) : `null`},
  null,
  null
)
  `.trim() + `\n`;
    result += `
function injectStyles (context) {
  for(let o in ${cssModuleVar}){
    this[o] = ${cssModuleVar}[o]
  }
}\n`;
    // TODO custom block
    // // Expose filename. This is used by the devtools and Vue runtime warnings.
    if (options.isProduction) {
        // Expose the file's full path in development, so that it can be opened
        // from the devtools.
        code += `\ncomponent.options.__file = ${JSON.stringify(path_1.default.relative(options.root, filePath).replace(/\\/g, '/'))}`;
    }
    // else if (options.exposeFilename) {
    // 	// Libraries can opt-in to expose their components' filenames in production builds.
    // 	// For security reasons, only expose the file's basename in production.
    // 	code += `\ncomponent.options.__file = ${JSON.stringify(filePath)}`
    // }
    if (options.devServer && !options.isProduction) {
        result += genHmrCode(descriptor.id, !!hasFunctional, templateRequest);
    }
    result += `\nexport default component.exports`;
    return result;
}
exports.transformMain = transformMain;
async function genScriptCode(scriptVar, descriptor, filename, options) {
    const { script } = descriptor;
    let scriptCode = `const ${scriptVar} = {}`;
    if (!script) {
        return { scriptCode };
    }
    let map;
    if (script) {
        // If the script is js/ts and has no external src, it can be directly placed
        // in the main module.
        if ((!script.lang || (script.lang === 'ts' && options.devServer)) &&
            !script.src) {
            scriptCode = compiler_sfc_1.rewriteDefault(script.content, scriptVar);
            map = script.map;
            if (script.lang === 'ts') {
                const result = await options.devServer.transformWithEsbuild(scriptCode, filename, { loader: 'ts' }, map);
                scriptCode = result.code;
                map = result.map;
            }
        }
        else {
            if (script.src) {
                linkSrcToDescriptor(script.src, filename, descriptor);
            }
            const src = script.src || filename;
            const langFallback = (script.src && path_1.default.extname(src).slice(1)) || 'js';
            const attrsQuery = attrsToQuery(script.attrs, langFallback);
            const srcQuery = script.src ? `&src` : ``;
            const query = `?vue&type=script${srcQuery}${attrsQuery}`;
            const request = JSON.stringify(src + query);
            scriptCode =
                `import ${scriptVar} from ${request}\n` + `export * from ${request}`; // support named exports
        }
    }
    return {
        scriptCode,
        map: map,
    };
}
function genTemplateRequest(filename, descriptor) {
    const template = descriptor.template;
    if (!template) {
        return { code: `const render, staticRenderFns` };
    }
    if (template.src) {
        linkSrcToDescriptor(template.src, filename, descriptor);
    }
    const src = template.src || filename;
    const srcQuery = template.src ? `&src` : ``;
    const attrsQuery = attrsToQuery(template.attrs, 'js', true);
    const query = `?vue&type=template${srcQuery}${attrsQuery}`;
    const templateRequest = src + query;
    return {
        code: `import { render, staticRenderFns } from '${templateRequest}'`,
        templateRequest,
    };
}
function genHmrCode(id, functional, templateRequest) {
    return `\n/* hot reload */
import __VUE_HMR_RUNTIME__ from "${index_1.vueHotReload}"
import vue from "vue"
__VUE_HMR_RUNTIME__.install(vue)
if(__VUE_HMR_RUNTIME__.compatible){
  if (!__VUE_HMR_RUNTIME__.isRecorded('${id}')) {
    __VUE_HMR_RUNTIME__.createRecord('${id}', component.options)
  }
   import.meta.hot.accept((update) => {
      __VUE_HMR_RUNTIME__.${functional ? 'rerender' : 'reload'}('${id}', update.default)
   })
   ${templateRequest
        ? `import.meta.hot.accept('${templateRequest}', (update) => {
      __VUE_HMR_RUNTIME__.rerender('${id}', update)
   })`
        : ''}
} else {
  console.log("The hmr is not compatible.")
}`;
}
function genStyleRequest(cssModuleVar, descriptor, filename) {
    let scoped = false;
    let stylesCode = '';
    descriptor.styles.forEach((style, i) => {
        if (style.src) {
            linkSrcToDescriptor(style.src, filename, descriptor);
        }
        const src = style.src || filename;
        const attrsQuery = attrsToQuery(style.attrs, 'css');
        const srcQuery = style.src ? `&src` : ``;
        const query = `?vue&type=style&index=${i}${srcQuery}`;
        const styleRequest = src + query + attrsQuery;
        if (style.scoped)
            scoped = true;
        if (style.module) {
            stylesCode += genCSSModulesCode(i, styleRequest, style.module, cssModuleVar);
        }
        else {
            stylesCode += `\nimport ${JSON.stringify(styleRequest)}`;
        }
    });
    return { scoped, stylesCode };
}
function genCSSModulesCode(index, request, moduleName, cssModuleVar) {
    const styleVar = `__style${index}`;
    const exposedName = typeof moduleName === 'string' ? moduleName : '$style';
    // inject `.module` before extension so vite handles it as css module
    const moduleRequest = request.replace(/\.(\w+)$/, '.module.$1');
    return (`\nimport ${styleVar} from ${JSON.stringify(moduleRequest)}` +
        `\n${cssModuleVar}["${exposedName}"] = ${styleVar}`);
}
/**
 * For blocks with src imports, it is important to link the imported file
 * with its owner SFC descriptor so that we can get the information about
 * the owner SFC when compiling that file in the transform phase.
 */
function linkSrcToDescriptor(src, filename, descriptor) {
    const srcFile = path_1.default.posix.resolve(path_1.default.posix.dirname(filename), src);
    descriptorCache_1.setDescriptor(srcFile, descriptor);
}
// these are built-in query parameters so should be ignored
// if the user happen to add them as attrs
const ignoreList = ['id', 'index', 'src', 'type', 'lang', 'module'];
function attrsToQuery(attrs, langFallback, forceLangFallback = false) {
    let query = ``;
    for (const name in attrs) {
        const value = attrs[name];
        if (!ignoreList.includes(name)) {
            query += `&${querystring_1.default.escape(name)}${value ? `=${querystring_1.default.escape(String(value))}` : ``}`;
        }
    }
    if (langFallback || attrs.lang) {
        query +=
            `lang` in attrs
                ? forceLangFallback
                    ? `&lang.${langFallback}`
                    : `&lang.${attrs.lang}`
                : `&lang.${langFallback}`;
    }
    return query;
}
//# sourceMappingURL=main.js.map